//////////////////////////////////////////////////
////////////////Texture Blending//////////////////
//////////////////////////////////////////////////
////07.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_Blend
#define OS_Blend


//blends two textures together with a factor and blends the result with another one using the alphachannel
MATERIAL* OS_Blend_ps20_mat =
{
	effect = "
	//Variables
		float4x4 matWorld;
		float4x4 matWorldViewProj;
		
		float4 vecFog;
		float4 vecTime;
		float4 vecColor;
		float4 vecViewDir;
		float4 vecViewPos;
		float4 vecSunDir;
		float4 vecSunColor;
		float4 vecLight;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecSkill1;
		float4 vecSkill5;
		
		
		//Textures
		texture mtlSkin1;
		texture mtlSkin2;
		texture mtlSkin3;
		
		
		//Sampler
		sampler blendMap1 = sampler_state
		{
			Texture = (mtlSkin1);
			AddressU = Wrap;
			AddressV = Wrap;
			
			MagFilter = Linear;
			MinFilter = Linear;
			MipFilter = Linear;
		};
		
		sampler blendMap2 = sampler_state
		{
			Texture = (mtlSkin2);
			AddressU = Wrap;
			AddressV = Wrap;
			
			MagFilter = Linear;
			MinFilter = Linear;
			MipFilter = Linear;
		};
		
		sampler blendMap3 = sampler_state
		{
			Texture = (mtlSkin3);
			AddressU = Wrap;
			AddressV = Wrap;
			
			MagFilter = Linear;
			MinFilter = Linear;
			MipFilter = Linear;
		};
		
		//Structs
		struct VS_TO_PS // Output to the pixelshader
		{
			float4 Pos	: POSITION;
			float  Fog	: FOG;
			
			float2 Tex0	: TEXCOORD0;
			float3 WPos	: TEXCOORD1;
			float3 Norm	: TEXCOORD2;
			
			float3 ViewDir : TEXCOORD3;
		};
		
		
		//Vertexshader
		VS_TO_PS VS_v0(	float4 inPos		: POSITION,
								float3 inNormal	: NORMAL,
								float2 inTex0		: TEXCOORD0	)
		{
			VS_TO_PS Out;
			
			//Do transformations
			Out.Pos = mul(inPos,matWorldViewProj);
			
			//Pass the Texcoords
			Out.Tex0 = inTex0;
			
			//Vertexposition in worldspace
			Out.WPos = mul(inPos,matWorld);
			
			//Vertexnormal
			Out.Norm = normalize(mul(inNormal,matWorld));
			
			//Direction to the view
			Out.ViewDir = normalize(Out.WPos-vecViewPos);
			
			//Fog
			Out.Fog = 1 - (distance(Out.WPos,vecViewPos)-vecFog.x) * vecFog.z;
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS_p0( VS_TO_PS In ) : COLOR
		{
			float3 LightDir;
			float3 Diffuse = vecLight.rgb+0.5+saturate(dot(-vecSunDir,In.Norm))*vecSunColor;
			float attenuation = 0;
			
			for(int i = 0; i < 2; i++)
			{
				LightDir = vecLightPos[i]-In.WPos;
				attenuation = saturate(1.0f - length(LightDir/vecLightPos[i].w));
				Diffuse += saturate(dot(normalize(LightDir),In.Norm))*vecLightColor[i]*attenuation;
			}
			
			float Glow = dot(In.ViewDir,In.Norm);
			Glow = saturate((pow(Glow+1,vecSkill1.z)-1)*vecSkill1.w);
			
			float4 ColorBlend1 = tex2D(blendMap1,In.Tex0);
			float4 ColorBlend2 = tex2D(blendMap2,In.Tex0);
			float4 ColorBlend3 = tex2D(blendMap3,In.Tex0);
			float4 Color = lerp(ColorBlend1,ColorBlend2,vecSkill1.x);
			Color = lerp(Color,ColorBlend3,vecSkill1.y*ColorBlend3.a);
			Color.rgb *= Diffuse;
			
			Color = lerp(Color,vecSkill5,Glow);
			Color.a = 1.0;
			
			return Color;
		}
		
		
		//////////////////////////////////////////////////////////////////
		technique Lighting
		{
			pass one
			{
				AlphaTestEnable = True;
				zWriteEnable = true;
				
				VertexShader = compile vs_1_1 VS_v0();
				PixelShader = compile ps_2_0 PS_p0();
			}
		}
		
		technique fallback { pass one { } }
	";
}


//initializes and activates the effect
void OS_Blend_init(ENTITY* Ent, BMAP* Map1, BMAP* Map2, BMAP* Map3, var BlendFactor1, var BlendFactor2, var BGLWidth, var BGLStrength, var BGLColRed, var BGLColGreen, var BGLColBlue)
{
	if(d3d_shaderversion >= 2020)
	{
		MATERIAL* Temp_Mat = mtl_create();
		Temp_Mat.effect = OS_Blend_ps20_mat.effect;
		Ent.material = Temp_Mat;
		
		Temp_Mat.skin1 = Map1;
		Temp_Mat.skin2 = Map2;
		Temp_Mat.skin3 = Map3;
		
		Temp_Mat.skill1 = floatd(BlendFactor1,100);
		Temp_Mat.skill2 = floatd(BlendFactor2,100);
		
		Temp_Mat.skill3 = floatv(BGLWidth);
		Temp_Mat.skill4 = floatv(BGLStrength);
		
		Temp_Mat.skill5 = floatd(BGLColRed,255);
		Temp_Mat.skill6 = floatd(BGLColGreen,255);
		Temp_Mat.skill7 = floatd(BGLColBlue,255);
	}
}

//function to change the material parameters after initializing
void OS_Blend_SetValues(ENTITY* Ent, BMAP* Map1, BMAP* Map2, BMAP* Map3, var BlendFactor1, var BlendFactor2, var BGLWidth, var BGLStrength, var BGLColRed, var BGLColGreen, var BGLColBlue)
{
	MATERIAL* Temp_Mat = Ent.material;
	
	Temp_Mat.skin1 = Map1;
	Temp_Mat.skin2 = Map2;
	Temp_Mat.skin3 = Map3;
	
	Temp_Mat.skill1 = floatd(BlendFactor1,100);
	Temp_Mat.skill2 = floatd(BlendFactor2,100);
	
	Temp_Mat.skill3 = floatv(BGLWidth);
	Temp_Mat.skill4 = floatv(BGLStrength);
	
	Temp_Mat.skill5 = floatd(BGLColRed,255);
	Temp_Mat.skill6 = floatd(BGLColGreen,255);
	Temp_Mat.skill7 = floatd(BGLColBlue,255);
}

#endif