//////////////////////////////////////////////////
////////////Per Pixel Lighting Shader/////////////
////////////- 5 Lights////////////////////////////
///////////////- with clipping////////////////////
//////////////////////////////////////////////////
////05.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_CI_PerPixelLighting
#define OS_CI_PerPixelLighting


void OS_CI_PerPixelLighting_mtl_event()
{
	mtl.skill20 = 0;
	
	mtl.skill14 = floatv(render_view.portal_x);
	mtl.skill15 = floatv(render_view.portal_y);
	mtl.skill16 = floatv(render_view.portal_z);
	
	mtl.skill17 = floatv(render_view.pnormal_x);
	mtl.skill18 = floatv(render_view.pnormal_y);
	mtl.skill19 = floatv(render_view.pnormal_z);
	
	#ifdef OS_Water
		if(render_view == OS_Water_Reflect_view || render_view == OS_Water_Refract_view)
		{
			mtl.skill20 = floatv(1);
		}
	#endif
}

//PS 2.0 Per Pixel Lighting for 5 lights
MATERIAL* OS_CI_PerPixelLighting_ps20_mat =
{
	flags = ENABLE_VIEW;
	event = OS_CI_PerPixelLighting_mtl_event;
	
	effect =
	"
		//Variables
		float4x4 matWorld;
		float4x4 matWorldViewProj;
		
		float4 vecViewDir;
		float4 vecViewPos;
		
		float4 vecFog;
		float4 vecTime;
		float4 vecColor;
		float4 vecLight;
		
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecSkill13;
		float4 vecSkill17;
		
		
		//Textures
		texture entSkin1;
		
		
		//Sampler
		sampler colorMap = sampler_state
		{
			Texture = (entSkin1);
			AddressU = Wrap;
			AddressV = Wrap;
			
			MagFilter = Linear;
			MinFilter = Linear;
			MipFilter = Linear;
		};
		
		//Structs
		struct VS_TO_PS // Output to the pixelshader
		{
			float4 Pos	: POSITION;
			float  Fog	: FOG;
			
			float3 Tex0	: TEXCOORD0;
			float3 WPos	: TEXCOORD1;
			float3 Norm	: TEXCOORD2;
		};
		
		
		//Vertexshader
		VS_TO_PS VS_v0(	float4 inPos		: POSITION,
								float3 inNormal	: NORMAL,
								float2 inTex0		: TEXCOORD0	)
		{
			VS_TO_PS Out;
			
			//Do transformations
			Out.Pos = mul(inPos,matWorldViewProj);
			
			//Pass the Texcoords
			Out.Tex0.xy = inTex0;
			
			//Vertexposition in worldspace
			Out.WPos = mul(inPos,matWorld);
			
			//Vertexnormal
			Out.Norm = normalize(mul(inNormal,matWorld));
			
			//Fog
			Out.Fog = 1 - (distance(Out.WPos,vecViewPos)-vecFog.x) * vecFog.z;
			
			//Clipping
			if(vecSkill17.w == 1.0)
			{
				Out.Tex0.z = (Out.WPos.y-vecSkill13.w)*vecSkill17.z;
			}
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS_p0( VS_TO_PS In ) : COLOR
		{
			//Clipping
			clip(In.Tex0.z);
			
			float3 LightDir;
			float3 Diffuse = vecLight+0.5;
			float attenuation = 0;
			
			for(int i = 0; i < 5; i++)
			{
				LightDir = vecLightPos[i]-In.WPos;
				attenuation = saturate(1.0f - length(LightDir/vecLightPos[i].w));
				Diffuse += saturate(dot(normalize(LightDir),In.Norm))*vecLightColor[i]*attenuation;
			}
			
			float4 Color = tex2D(colorMap,In.Tex0.xy);
			Color.rgb *= Diffuse;
			
//			Color.a = step(0.5,Color.a);
			
			return Color;
		}
		
		
		//////////////////////////////////////////////////////////////////
		technique Lighting
		{
			pass one
			{
				AlphaTestEnable = True;
				zWriteEnable = true;
				
				VertexShader = compile vs_1_1 VS_v0();
				PixelShader = compile ps_2_0 PS_p0();
			}
		}
		
		technique fallback { pass one { } }
	";
}


//function to add the Shader to a model
void OS_CI_PerPixelLighting_init(ENTITY* Ent)
{
	if(d3d_shaderversion >= 2020)
	{
		if(Ent != NULL)
		{
			OS_Mtl_set(Ent,OS_CI_PerPixelLighting_ps20_mat);
		}else
		{
			mat_model.effect = OS_CI_PerPixelLighting_ps20_mat.effect;
			set(mat_model,ENABLE_VIEW);
			mat_model.event = OS_CI_PerPixelLighting_mtl_event;
		}
	}
}

#endif