//////////////////////////////////////////////////
///////////////////Heat Haze//////////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_HeatHaze
#define OS_HeatHaze

#ifndef OS_Shared_Refract
	#include "OS_Shared_Refract.c"
#endif


//disorts the refraction view
MATERIAL* OS_HeatHaze_ps20_mat =
{
	flags = ENABLE_RENDER;
	event = OS_HeatHaze_mtl_event;
	
	effect = "
	float4x4 matWorld;
	float4x4 matWorldViewProj;
	
	float4 vecTime;
	float4 vecViewPos;
	float4 vecSkill1;
	float4 vecSkill41;
	
	Texture entSkin1;
	Texture entSkin2;
	Texture entSkin3;
	
	sampler2D smpSource = sampler_state
	{
		texture=<entSkin1>;
		AddressU  = clamp;
	   AddressV  = clamp;
	};
	sampler2D smpBump = sampler_state{texture=<entSkin2>;};
	sampler2D smpBlend = sampler_state{texture=<entSkin3>;};
	
	//Structs
	struct VS_TO_PS // Output to the pixelshader
	{
		float4 Pos	: POSITION;
		
		float4 Tex		: TEXCOORD0;
		float3 ProjTex	: TEXCOORD1;
	};
	
	
	//Vertexshader
	VS_TO_PS VS(float4 inPos		: POSITION,
					float2 inTex0		: TEXCOORD0)
	{
		VS_TO_PS Out;
		
		//do the transformations
		Out.Pos = mul(inPos,matWorldViewProj);
		
		//tex coords
		Out.Tex.xy = inTex0;
		
		//bump coordinates
		Out.Tex.zw = inTex0+vecSkill41.zw*vecTime.w;
		
		Out.ProjTex = Out.Pos.xyw;
		
		return Out;
	}
	
	
	//Pixelshader
	float4 PS(VS_TO_PS In):COLOR
	{
		clip(vecSkill1.x);
		
		float2 Bump = tex2D(smpBump,In.Tex.zw).rg*2-1.0;
		Bump *= vecSkill41.y;
		
		float2 Blend = tex2D(smpBlend,In.Tex.xy).rg;
		
		//get the projection coords
		float2 temp = In.ProjTex.xy/In.ProjTex.z;
		float2 ProjCoord = 0.5+0.5*temp+Bump*Blend.y;
		
		return float4(tex2D(smpSource,ProjCoord).rgb,Blend.x);
	}
	
	
	//Technique
	technique Refraction
	{
		pass one
		{
//			AlphaTestEnable = True;
//			AlphaBlendEnable = True;
			
			VertexShader = compile vs_1_1 VS();
			PixelShader = compile ps_2_0 PS();
		}
	}
	";
}

//the materials event to prevent it from beeing rendered within its refraction view
void OS_HeatHaze_mtl_event()
{
	if(render_view == OS_Shared_Refract_view)
	{
		mtl.skill1 = floatv(-1);
	}else
	{
		mtl.skill1 = floatv(1);
	}
}


//initializes and activates the effect
void OS_HeatHaze_init(ENTITY* Ent, BMAP* RefractMap, BMAP* BlendMap, var Resolution, var Strength, var SpeedX, var SpeedY)
{
	if(d3d_shaderversion >= 2020)
	{
		OS_Mtl_set(Ent,OS_HeatHaze_ps20_mat);
		
		OS_Shared_RefractView_init(Resolution,Resolution);
		
		set(Ent,FLAG1|CAST|TRANSLUCENT);
		
		ent_setskin(Ent, OS_Shared_Refract_bmap, 1);
		ent_setskin(Ent, bmap_to_mipmap(RefractMap), 2);
		ent_setskin(Ent, bmap_to_mipmap(BlendMap), 3);
		
		Ent.skill42 = floatd(Strength,100);
		Ent.skill43 = floatv(SpeedX);
		Ent.skill44 = floatv(SpeedY);
	}
}

#endif