//////////////////////////////////////////////////
///////////////////Heat Haze//////////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_Ice
#define OS_Ice

#ifndef OS_Shared_Refract
	#include "OS_Shared_Refract.c"
#endif


//disorts the refraction view
MATERIAL* OS_Ice_ps20_mat =
{
	flags = ENABLE_RENDER, TRANSLUCENT;
	event = OS_Ice_mtl_event;
	
	effect = "
	float4x4 matWorld;
	float4x4 matWorldViewProj;
	
	float4 vecTime;
	float4 vecViewPos;
	float4 vecSkill1;
	float4 vecSkill41;
	float4 vecLight;
	float4 vecSunDir;
	float4 vecSunColor;
	
	texture entSkin1;
	texture entSkin2;
	texture entSkin3;
	texture entSkin4;
	
	sampler2D smpSource = sampler_state
	{
		texture = <entSkin1>;
		AddressU  = clamp;
	   AddressV  = clamp;
	};

	sampler2D smpBump = sampler_state{texture=<entSkin2>;};
	
	samplerCUBE EnvironmentSampler = sampler_state
	{ 
	    Texture = <entSkin3>;
	    MipFilter = LINEAR;
	    MinFilter = LINEAR;
	    MagFilter = LINEAR;
	};

	sampler2D smpColor = sampler_state{texture=<entSkin4>;};
	
	//Structs
	struct VS_TO_PS // Output to the pixelshader
	{
		float4 Pos	: POSITION;
		
		float2 Tex		: TEXCOORD0;
		float3 ProjTex	: TEXCOORD1;
		float3 View		: TEXCOORD2;
		float3 Normal	: TEXCOORD3;
	};
	
	
	//Vertexshader
	VS_TO_PS VS(float4 inPos		: POSITION,
					float3 inNormal	: NORMAL,
					float2 inTex0		: TEXCOORD0)
	{
		VS_TO_PS Out;
		
		//do the transformations
		Out.Pos = mul(inPos,matWorldViewProj);
		
		Out.Normal = normalize(mul(float4(inNormal,1.0),matWorld));
		
		float4 Pos = mul(inPos,matWorld);
		Out.View = normalize(Pos - vecViewPos);
		
		//tex coords
		Out.Tex.xy = inTex0;
		
		Out.ProjTex = Out.Pos.xyw;
		
		return Out;
	}
	
	
	//Pixelshader
	float4 PS(VS_TO_PS In):COLOR
	{
		clip(vecSkill1.x);
		
		float2 Bump = tex2D(smpBump,In.Tex.xy).rg*2-1.0;
		Bump *= vecSkill41.y;
		
		float4 ColorMap = tex2D(smpColor,In.Tex.xy);
		
		float3 CubeTexcoord = reflect(In.View,In.Normal+float3(Bump.rg,0));
		float3 Environment = texCUBE(EnvironmentSampler,CubeTexcoord).rgb;
		
		//get the projection coords
		float2 temp = In.ProjTex.xy/In.ProjTex.z;
		float2 ProjCoord = 0.5+0.5*temp+Bump;
		
		float3 Diffuse = vecLight.rgb+0.5+saturate(dot(-vecSunDir,In.Normal))*vecSunColor;
		
		float3 Color = lerp(Environment,tex2D(smpSource,ProjCoord).rgb,0.5);
		Color = lerp(ColorMap.rgb,Color,ColorMap.a)*Diffuse;
		return float4(Color,1.0f);
	}
	
	
	//Technique
	technique Refraction
	{
		pass P0
		{
			VertexShader = compile vs_1_1 VS();
			PixelShader = compile ps_2_0 PS();
		}
	}
	";
}

//the materials event to prevent it from beeing rendered within its refraction view
void OS_Ice_mtl_event()
{
	if(render_view == OS_Shared_Refract_view)
	{
		mtl.skill1 = floatv(-1);
	}else
	{
		mtl.skill1 = floatv(1);
	}
}


//initializes and activates the effect
void OS_Ice_init(ENTITY* Ent, BMAP* ColorMap, BMAP* RefractMap, BMAP* CubeMap, var Resolution, var Strength)
{
	if(d3d_shaderversion >= 2020)
	{
		OS_Mtl_set(Ent,OS_Ice_ps20_mat);
		
		OS_Shared_RefractView_init(Resolution,Resolution);
		
		set(Ent,FLAG1|FLAG2|CAST);
		
		ent_setskin(Ent, OS_Shared_Refract_bmap, 1);
		ent_setskin(Ent, bmap_to_mipmap(RefractMap), 2);
		ent_setskin(Ent, bmap_to_cubemap(CubeMap), 3);
		ent_setskin(Ent, bmap_to_mipmap(ColorMap), 4);
		
		Ent.skill42 = floatd(Strength,100);
	}
}

#endif