//////////////////////////////////////////////////
////////////////Multitextureshader////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_MultiTex
#define OS_MultiTex


//Multitextureshader for 4 textures and a shadowmap for PS 2.0 supporting 4 lights on a per pixel basis
MATERIAL* OS_MultiTex_ps20_mat =
{
	effect =
	"
		//Variables
		float4x4 matWorldViewProj;
		float4x4 matWorld;
		
		float4 vecFog;
		float4 vecViewPos;
		
		float3 vecSunDir;
		float4 vecSunColor;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		float4 vecLight;
		
		float4 vecSkill1;
		
		
		//Textures
		Texture entSkin1;
		Texture entSkin2;
		Texture entSkin3;
		Texture entSkin4;
		Texture mtlSkin1;
		Texture mtlSkin2;
		
		
		//Samplers
		sampler mapBlend = sampler_state
		{
			Texture = <entSkin1>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		sampler mapShadow = sampler_state
		{
			Texture = <entSkin2>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		sampler mapS = sampler_state
		{
			Texture = <entSkin3>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		sampler mapR = sampler_state
		{
			Texture = <entSkin4>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		sampler mapG = sampler_state
		{
			Texture = <mtlSkin1>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		sampler mapB = sampler_state
		{
			Texture = <mtlSkin2>;
			MipFilter = LINEAR;
			MinFilter = LINEAR;
			MagFilter = LINEAR;
			AddressU = Wrap;
			Addressv = Wrap;
		};
		
		
		//Structs
		struct VS_TO_PS
		{
			float4 Pos			: POSITION;
			float Fog			: FOG;
			
			float3 Normal		: TEXCOORD1;
			
			float2 TexCoords	: TEXCOORD2;
			float2 TexCoords2	: TEXCOORD3;
			
			float3 WPos			: TEXCOORD4;
		};
		
		
		//Vertexshader
		VS_TO_PS TMulti_VS(	float4 inPos		: POSITION,
									float4 inNormal	: NORMAL,
									float2 TexCoords	: TEXCOORD0	)
		{
			VS_TO_PS Out;
			
			//do the transformations
			Out.Pos = mul(inPos, matWorldViewProj);
			
			//pass the normal
			Out.Normal = inNormal;	// It somehow does not work if it is multiplicated with matWorld!!!
			
			
			//Fog
			Out.WPos = mul(inPos, matWorld);
			Out.Fog = 1 - (distance(Out.WPos, vecViewPos) - vecFog.x) * (vecFog.z);
			
			//Pass the texture coordinates
			Out.TexCoords.xy = TexCoords;
			Out.TexCoords2.xy = TexCoords*vecSkill1.x;
			
			return Out;
		}
		
		
		//Pixelshader
		float4 TMulti_PS(VS_TO_PS In): COLOR
		{
			//get blend and shadow texture
			float4 BlendColor = tex2D(mapBlend, In.TexCoords.xy);
			float4 ShadowColor = tex2D(mapShadow,In.TexCoords.xy);
			
			//get the tiled textures
			float4 SColor = tex2D(mapS, In.TexCoords2.xy);
			float4 RColor = tex2D(mapR, In.TexCoords2.xy);
			float4 GColor = tex2D(mapG, In.TexCoords2.xy);
			float4 BColor = tex2D(mapB, In.TexCoords2.xy);
			
			//blend the textures together
			float4 Color = SColor;
			Color = lerp(Color, RColor, BlendColor.r);
			Color = lerp(Color, GColor, BlendColor.g);
			Color = lerp(Color, BColor, BlendColor.b);
			
			//compute the diffuse shading
			float4 Diffuse = (vecLight/*+saturate(dot(-vecSunDir, In.Normal))*vecSunColor*/)*ShadowColor;
			float3 LightDir;
			float attenuation;
			
			for(int i = 0; i < 4; i++)
			{
				LightDir = vecLightPos[i]-In.WPos;
				attenuation = saturate(1.0f - length(LightDir/vecLightPos[i].w));
				Diffuse += saturate(dot(normalize(LightDir),In.Normal))*vecLightColor[i]*attenuation;
			}
			
			//compute the final color
			Color = Color*Diffuse;
			Color.a = 1.0f;
			
			return Color;
		}
		
		
		//Technique
		technique tmulti
		{
			pass one
			{
				VertexShader = compile vs_1_1 TMulti_VS();
				PixelShader = compile ps_2_0 TMulti_PS();
			}
		}
	";
}

//ffp detailmapping with shadowmap
MATERIAL* OS_MultiTex_ffp_mat =
{
	effect =
	"
		//Variables
		float4x4 matMtl;
		
		float4 vecLight;
		
		//Textures
		texture entSkin1;
		texture entSkin2;
		texture entSkin3;
		
		//Technique
		technique t0
		{
			pass p0
			{
				Texture[0] = <entSkin1>;	//Colormap
				Texture[1] = <entSkin3>;	//Detailmap
				Texture[2] = <entSkin2>;	//Shadowmap
				
				TextureFactor = <vecLight>;
				
				//Color
				colorarg1[0] = Texture;
				colorop[0] = selectarg1;
				texcoordindex[0] = 0;
				
				//add the Detailmap
				colorarg1[1] = Texture;
				colorarg2[1] = current;
				colorop[1] = addsigned;
				texcoordindex[1] = 0;
				texturetransformflags[1] = count2;
				texturetransform[1] = <matMtl>;
				
				//add the Shadowmap
				colorarg1[2] = Texture;
				colorarg2[2] = current;
				colorop[2] = modulate;	
				texcoordindex[2] = 0;
				
				//add diffuse lighting
				colorarg1[3] = Diffuse;
				colorarg2[3] = current;
				colorop[3] = modulate;
				
				//add ambient lighting
				colorarg1[4] = TFactor;
				colorarg2[4] = current;
				colorop[4] = Add;
			}
		}
	";
}

//assigns the shader to the entity
void OS_MultiTex_init(ENTITY* Ent, BMAP* BlendMap, BMAP* ShadowMap, BMAP* BlackMap, BMAP* RedMap, BMAP* GreenMap, BMAP* BlueMap, BMAP* FallbackMap, BMAP* FallbackDetailMap, var Tilefactor)
{
	MATERIAL* Temp_Mat = mtl_create();
	
	if(d3d_shaderversion >= 2020)
	{
		
		Temp_Mat.effect = OS_MultiTex_ps20_mat.effect;
		Ent.material = Temp_Mat;
		
		bmap_to_mipmap(BlendMap);
		bmap_to_mipmap(ShadowMap);
		bmap_to_mipmap(BlackMap);
		bmap_to_mipmap(RedMap);
		bmap_to_mipmap(GreenMap);
		bmap_to_mipmap(BlueMap);
		
		ent_setskin(Ent,BlendMap,1);
		ent_setskin(Ent,ShadowMap,2);
		ent_setskin(Ent,BlackMap,3);
		ent_setskin(Ent,RedMap,4);
		Temp_Mat.skin1 = GreenMap;
		Temp_Mat.skin2 = BlueMap;
		
		Temp_Mat.skill1 = floatv(Tilefactor);
	}else
	{
		Temp_Mat.effect = OS_MultiTex_ffp_mat.effect;
		Ent.material = Temp_Mat;
		
		bmap_to_mipmap(FallbackMap);
		bmap_to_mipmap(ShadowMap);
		bmap_to_mipmap(FallbackDetailMap);
		
		ent_setskin(Ent,FallbackMap,1);
		ent_setskin(Ent,ShadowMap,2);
		ent_setskin(Ent,FallbackDetailMap,3);
		
		mat_identity(Temp_Mat.matrix);
		Temp_Mat.matrix11 = Tilefactor;
		Temp_Mat.matrix22 = Tilefactor;
	}
}

#endif