//////////////////////////////////////////////////
///////////////Normalmappingshader////////////////
///////////////- Sun and 3 Lights/////////////////
//////////////////////////////////////////////////
////03.05.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////

#ifndef OS_O_Normalmapping
#define OS_O_Normalmapping


//Normalmappingshader for PS 2.0 supporting 3 lights and the sun
MATERIAL* OS_O_Normalmapping_ps20_mat =
{
	effect =
	"
		//Variables
		float4x4 matWorldViewProj;	
		float4x4 matWorld;
		float3x3 matTangent;
		
		float4 vecSunDir;
		float4 vecSunColor;
		float4 vecLightPos[8];
		float4 vecLightColor[8];
		
		float4 vecViewPos;
		
		float4 vecFog;
		float4 vecLight;
		
		float4 vecSkill41;
		
		
		//Textures
		texture entSkin1; //this is the color map
		texture entSkin2; //this is the normal map
		
		
		//Sampler
		sampler colorMap = sampler_state
		{
		   Texture = <entSkin1>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		sampler bumpMap = sampler_state
		{
		   Texture = <entSkin2>;
		   MinFilter = Linear;
		   MagFilter = Linear;
		   MipFilter = Linear;   
		   AddressU  = wrap;
		   AddressV  = wrap;
		};
		
		
		//Structs
		struct VS_TO_PS
		{
			float4 Pos		: POSITION;
			float Fog		: FOG;
			
			float2 Tex		: TEXCOORD0;
			
			float3 View		: TEXCOORD1;
			float3 Sun		: TEXCOORD2;
			
			float4 Light1	: TEXCOORD3;
			float4 Light2	: TEXCOORD4;
			float4 Light3	: TEXCOORD5;
		};
		
		
		//Vertexshader
		VS_TO_PS VS(float4 inPos		: POSITION,
						float2 inTex		: TEXCOORD0,
						float3 inNormal	: NORMAL,
						float4 inTangent	: TEXCOORD2	)
		{
			VS_TO_PS Out = (VS_TO_PS)0;
			
			//Calculate the final vertexposition
			Out.Pos = mul(inPos, matWorldViewProj);
			
			//Get the vertexposition in worldspace
			float3 PosWorld = mul(inPos, matWorld);
			
			//Create matTangent
			matTangent[0] = mul(inTangent.xyz, matWorld);
			matTangent[1] = mul(cross(inTangent.xyz, inNormal)*inTangent.w, matWorld);
			matTangent[2] = mul(inNormal, matWorld);
			
			//Pass the TexCoords
			Out.Tex = inTex;
			
			//Viewdir to tangent space
			Out.View = mul(matTangent, vecViewPos.xyz-PosWorld.xyz);
			
			//Sundir to tangent space
			Out.Sun = normalize(mul(matTangent, -vecSunDir));
			
			float3 LightDir = 0;
			
			//Light1
			LightDir = vecLightPos[0]-PosWorld;
			Out.Light1.xyz = normalize(mul(matTangent,LightDir));
			Out.Light1.w = saturate(1.0f - length(LightDir/vecLightPos[0].w));
			
			//Light2
			LightDir = vecLightPos[1]-PosWorld;
			Out.Light2.xyz = normalize(mul(matTangent,LightDir));
			Out.Light2.w = saturate(1.0f - length(LightDir/vecLightPos[1].w));
			
			//Light3
			LightDir = vecLightPos[2]-PosWorld;
			Out.Light3.xyz = normalize(mul(matTangent,LightDir));
			Out.Light3.w = saturate(1.0f - length(LightDir/vecLightPos[2].w));
			
			//Fog
			Out.Fog = 1 - (distance(PosWorld,vecViewPos)-vecFog.x) * vecFog.z;
			
			return Out;
		}
		
		
		//Pixelshader
		float4 PS(VS_TO_PS In) : COLOR
		{
			float4 Color = tex2D(colorMap,In.Tex);
			float4 Bump = tex2D(bumpMap,In.Tex);
			Bump.rgb = Bump.rgb * 2.0f - 1.0f;
			
			Bump.a *= vecSkill41.x;
			
			In.View = normalize(In.View);
			
			//Sun
			float3 SunDiff = saturate(dot(Bump.rgb,In.Sun));
			float3 SunReflect = normalize(2 * SunDiff * Bump.rgb - In.Sun);
			float3 SunSpec = pow(saturate(dot(SunReflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Sun = (Color.rgb*SunDiff+SunSpec) * vecSunColor;
			
			//Light1
			float3 Light1Diff = saturate(dot(Bump.rgb,In.Light1.xyz));
			float3 Light1Reflect = normalize(2 * Light1Diff * Bump.rgb - In.Light1.xyz);
			float3 Light1Spec = pow(saturate(dot(Light1Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light1 = (Color.rgb*Light1Diff+Light1Spec) * vecLightColor[0] * In.Light1.w;
			
			//Light2
			float3 Light2Diff = saturate(dot(Bump.rgb,In.Light2.xyz));
			float3 Light2Reflect = normalize(2 * Light2Diff * Bump.rgb - In.Light2.xyz);
			float3 Light2Spec = pow(saturate(dot(Light2Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light2 = (Color.rgb*Light2Diff+Light2Spec) * vecLightColor[1] * In.Light2.w;
			
			//Light3
			float3 Light3Diff = saturate(dot(Bump.rgb,In.Light3.xyz));
			float3 Light3Reflect = normalize(2 * Light3Diff * Bump.rgb - In.Light3.xyz);
			float3 Light3Spec = pow(saturate(dot(Light3Reflect,In.View)), vecSkill41.y)*Bump.a;
			float3 Light3 = (Color.rgb*Light3Diff+Light3Spec) * vecLightColor[2] * In.Light3.w;
			
			//Compute the final color
			Color.rgb = Color.rgb*(vecLight.rgb)+Sun+Light1+Light2+Light3;
			
			return Color;
		}
		
		
		//Technique
		technique NormalMapping
		{
			pass P0
			{
				alphatestenable = true;
				zWriteEnable = true;
				
				// compile shaders
				VertexShader = compile vs_1_1 VS();
				PixelShader  = compile ps_2_0 PS();
			}
		}
	";
}

//assigns the shader to the entity
void OS_O_Normalmapping_init(ENTITY* Ent, var SpecStrength, var SpecIntensity)
{
	if(d3d_shaderversion >= 2020)
	{
		OS_Mtl_set(Ent,OS_O_Normalmapping_ps20_mat);
		
		Ent.skill41 = floatv(SpecStrength);
		Ent.skill42 = floatv(SpecIntensity);
	}
}

#endif