//////////////////////////////////////////////////
///////////////Postprocessing Bloom///////////////
//////////////////////////////////////////////////
//////////based on an example by Scorpion/////////
//////////////////////////////////////////////////
////09.03.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_Bloom


//material which makes dark parts black
MATERIAL* PP_Bloom_Blackout_mat =
{
	effect =
	"
		Texture TargetMap;
		sampler2D smpSource = sampler_state
		{
			texture = <TargetMap>;
		};
		
		float4 vecViewPort; // contains viewport pixel size in zw components
		float4 vecSkill1;
		
		float4 BlackoutPS( float2 Tex : TEXCOORD0 ) : COLOR0 
		{
			float4 Color = tex2D(smpSource,Tex.xy);
			if(dot(Color.rgb,float3(0.299f,0.587f,0.114f)) < vecSkill1.x)
			{
				Color.rgb = 0;
			}
			return Color;
		}

		technique blackout
		{
			pass one
			{
				PixelShader = compile ps_2_0 BlackoutPS();
			}
		}
	";
}


//a horizontal blur
MATERIAL* PP_Bloom_HBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurHPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			Tex.xy *= vecSkill1.y;
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.y = Tex.y;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.x = Tex.x+i*vecViewPort.z*vecSkill1.x*vecSkill1.y;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurHPS();}}
		";
}


//a vertical blur
MATERIAL* PP_Bloom_VBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurVPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.x = Tex.x;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.y = Tex.y+i*vecViewPort.w*vecSkill1.x;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurVPS();}}
		";
}


//combines the blurred and the normal image to the final bloomed one
MATERIAL* PP_Bloom_Combine_mat = 
{
	effect = "
	float4 vecViewPort;
	float4 vecSkill1;
	
	Texture TargetMap;
	Texture mtlSkin1;
	sampler2D smpSource = sampler_state{texture=<TargetMap>;};
	sampler2D firstScreen = sampler_state{texture=<mtlSkin1>;};
	
	float4 AddPS(	float2 Tex: TEXCOORD0):COLOR0
	{
		float4 Color = tex2D(firstScreen,Tex);
		Color += vecSkill1.x*tex2D(smpSource,Tex*vecSkill1.y);
		return Color;
	}
	technique Blackout { pass one { PixelShader = compile ps_2_0 AddPS();}}
	";
}


//function to change the values for bloom
void PP_Bloom_SetValues(var BlackoutBrightness, var BlurRadius, var BlurAlpha, var Downsample)
{
	//set the effects parameter
	PP_Bloom_Blackout_mat.skill1 = floatv((100-BlackoutBrightness)/100);
	PP_Bloom_HBlur_mat.skill1 = floatv(BlurRadius/100);
	PP_Bloom_HBlur_mat.skill2 = floatv(Downsample);
	PP_Bloom_VBlur_mat.skill1 = PP_Bloom_HBlur_mat.skill1;
	PP_Bloom_Combine_mat.skill1 = floatv(BlurAlpha/100);
	PP_Bloom_Combine_mat.skill2 = floatd(1,Downsample);
}


//initializes and activates bloom
void PP_Bloom_init(VIEW* View, var BlackoutBrightness, var BlurRadius, var BlurAlpha, var Downsample)
{
	if(d3d_shaderversion >= 2020)
	{
		//set the effects parameter
		PP_Bloom_SetValues(BlackoutBrightness,BlurRadius,BlurAlpha,Downsample);
		
		//store the previous stages image
		View = PP_LastView(View);
		if(PP_Bloom_Combine_mat.skin1 == NULL){PP_Bloom_Combine_mat.skin1 = bmap_createblack(screen_size.x,screen_size.y,24);}
		View.bmap = PP_Bloom_Combine_mat.skin1;
		
		//create the effectchain
		PP_Add(PP_Bloom_Blackout_mat,View,NULL);
		PP_Add(PP_Bloom_HBlur_mat,View,NULL);
		PP_Add(PP_Bloom_VBlur_mat,View,NULL);
		PP_Add(PP_Bloom_Combine_mat,View,NULL);
	}
}

//removes Bloom
void PP_Bloom_remove(VIEW* View)
{
	//remove the stages
	PP_Remove(PP_Bloom_Blackout_mat,View,NULL);
	PP_Remove(PP_Bloom_HBlur_mat,View,NULL);
	PP_Remove(PP_Bloom_VBlur_mat,View,NULL);
	PP_Remove(PP_Bloom_Combine_mat,View,NULL);
	
	//remove the bmap
	ptr_remove(PP_Bloom_Combine_mat.skin1);
	PP_Bloom_Combine_mat.skin1 = NULL;
}

//makes it fit to the new resolution
void PP_Bloom_Video_Switch(VIEW* View)
{
	//if Bloom is active
	if(PP_Bloom_Combine_mat.skin1 != NULL)
	{
		//remove the effect
		PP_Bloom_remove(View);
		
		//and create it again:
		
		//store the previous stages image
		View = PP_LastView(View);
		if(PP_Bloom_Combine_mat.skin1 == NULL){PP_Bloom_Combine_mat.skin1 = bmap_createblack(screen_size.x,screen_size.y,24);}
		View.bmap = PP_Bloom_Combine_mat.skin1;
		
		//create the effectchain
		PP_Add(PP_Bloom_Blackout_mat,View,NULL);
		PP_Add(PP_Bloom_HBlur_mat,View,NULL);
		PP_Add(PP_Bloom_VBlur_mat,View,NULL);
		PP_Add(PP_Bloom_Combine_mat,View,NULL);
	}
}