//////////////////////////////////////////////////
////////////////Postprocessing Blur///////////////
//////////////////////////////////////////////////
////07.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_Blur


//a horizontal blur
MATERIAL* PP_Blur_HBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurHPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			Tex.xy *= vecSkill1.x;
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.y = Tex.y;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.x = Tex.x+i*vecViewPort.z*vecSkill1.y*vecSkill1.x;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurHPS();}}
		";
}

//a vertical blur
MATERIAL* PP_Blur_VBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurVPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.x = Tex.x;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.y = Tex.y+i*vecViewPort.w*vecSkill1.x;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurVPS();}}
	";
}

//upsamples the blurred image
MATERIAL* PP_Blur_Upsample_mat =
{
	effect = "
		float4 vecSkill1;
		
		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};
		
		float4 BlurVPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			return tex2D(smpSource,Tex.xy*vecSkill1.x);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurVPS();}}
	";
}

//change the blurs parameters
void PP_Blur_SetValues(var Downsample, var HBlur, var HBlur)
{
	PP_Blur_HBlur_mat.skill1 = floatv(Downsample);
	PP_Blur_HBlur_mat.skill2 = floatd(HBlur,100);
	PP_Blur_VBlur_mat.skill1 = floatd(HBlur,100);
	PP_Blur_Upsample_mat.skill1 = floatd(1,Downsample);
}

//activate the blur
void PP_Blur_init(VIEW* View, var Downsample, var HBlur, var HBlur)
{
	if(d3d_shaderversion >= 2020)
	{
		//pass the settings to the shader
		PP_Blur_SetValues(Downsample,HBlur,HBlur);
		
		//create the new stages
		PP_Add(PP_Blur_HBlur_mat,View,NULL);
		PP_Add(PP_Blur_VBlur_mat,View,NULL);
		PP_Add(PP_Blur_Upsample_mat,View,NULL);
	}
}

//removes the blur
void PP_Blur_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_Blur_HBlur_mat,View,NULL);
	PP_Remove(PP_Blur_VBlur_mat,View,NULL);
	PP_Remove(PP_Blur_Upsample_mat,View,NULL);
}