//////////////////////////////////////////////////
///////////////////Heat Haze//////////////////////
//////////////////////////////////////////////////
////02.01.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_HeatHaze

#ifndef PP_Shared_Depth
	#include "PP_Shared_Depth.c"
#endif


//disorts the screen depending on the depth
MATERIAL* PP_HeatHaze_mat = 
{
	effect = "
	#define PP_Shared_Depth_Precision 10000
	
	float4 vecViewPort;
	float4 vecTime;
	float4 vecSkill1;
	
	Texture TargetMap;
	Texture mtlSkin1;
	Texture mtlSkin2;
	sampler2D smpSource = sampler_state{texture=<TargetMap>;};
	sampler2D smpDepth = sampler_state{texture=<mtlSkin1>;};
	sampler2D smpBump = sampler_state{texture=<mtlSkin2>;};
	
	float4 AddPS(	float2 Tex: TEXCOORD0):COLOR0
	{
		float3 FracDepth = tex2D(smpDepth,Tex).rgb;
		float depth = dot(FracDepth.rgb,float3(255.0/256,255.0/(256*256),255.0/(256*256*256)))*PP_Shared_Depth_Precision;
		
		float BlendDepth = 0.0;
		if(depth < vecSkill1.x)
		{
			BlendDepth = 0;
		}else
		{
			BlendDepth = (depth-vecSkill1.x)/vecSkill1.y;
		}
		BlendDepth = saturate(BlendDepth);
		
		float2 Bump = tex2D(smpBump,Tex+float2(0,vecTime.w*vecSkill1.w)).rg*2-1.0;
		Tex += Bump*BlendDepth*vecSkill1.z;
		
		return tex2D(smpSource,Tex);
	}
	technique Blackout { pass one { PixelShader = compile ps_2_0 AddPS();}}
	";
}


//function to change the values for HeatHaze
void PP_HeatHaze_SetValues(BMAP* NormalMap, var StartDist, var EndDist, var Strength, var Speed)
{
	//set the skin
	PP_HeatHaze_mat.skin2 = NormalMap;
	
	//set the effects parameter
	PP_HeatHaze_mat.skill1 = floatv(StartDist);
	PP_HeatHaze_mat.skill2 = floatv(EndDist);
	PP_HeatHaze_mat.skill3 = floatd(Strength,100);
	PP_HeatHaze_mat.skill4 = floatv(Speed);
}


//initializes and activates HeatHaze
void PP_HeatHaze_init(VIEW* View, BMAP* NormalMap, var StartDist, var EndDist, var Strength, var Speed)
{
	if(d3d_shaderversion >= 2020)
	{
		//set the effects parameter
		PP_HeatHaze_SetValues(NormalMap,StartDist,EndDist,Strength,Speed);
		
		//create the effectchain
		PP_Add(PP_HeatHaze_mat,View,NULL);
		
		//assign the depthmap to the shader
		if(PP_Shared_Depth_orig_view == NULL)
		{
			PP_Shared_Depth_init(View);
		}
		PP_HeatHaze_mat.skin1 = PP_Shared_Depth_orig_view.bmap;
	}
}

//removes HeatHaze
void PP_HeatHaze_remove(VIEW* View)
{
	//remove the stages
	PP_Remove(PP_HeatHaze_mat,View,NULL);
	
	//set the texture to NULL
	PP_HeatHaze_mat.skin2 = NULL;
}