//////////////////////////////////////////////////
/////////////Postprocessing Old Movie/////////////
//////////////////////////////////////////////////
////18.03.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_OldMovie


//an old movie shader, disorting the colors a bit and adding grain and a scratch
MATERIAL* PP_OldMovie_mat =
{
	effect =
	"
		texture TargetMap;
		sampler2D smpSource = sampler_state
		{
			texture = <TargetMap>;
			AddressU = Clamp;
			AddressV = Clamp;
		};
		
		texture mtlSkin1;
		sampler2D smpGrain = sampler_state
		{
			texture = <mtlSkin1>;
			AddressU = Wrap;
			AddressV = Wrap;
		};
		
		texture mtlSkin2;
		sampler1D smpScratch = sampler_state
		{
			texture = <mtlSkin2>;
			AddressU = Wrap;
		};
		
		float4 vecViewPort; // contains viewport pixel size in zw components
		float4 vecSkill1;	//blurstrength,saturation,sepiastrength,darken
		float4 vecSkill5;	//contrast,scratchspeed
		float4 vecTime;
		
		float4 OldMoviePS( float2 Tex : TEXCOORD0 ) : COLOR0 
		{
			float4 Color = 0;
			
			float BlurDist = 1;
			for (float i=-BlurDist; i<=BlurDist; i+=BlurDist)
				for (float j=-BlurDist; j<=BlurDist; j+=BlurDist)
					Color += tex2D(smpSource,Tex.xy+float2(i*vecViewPort.z,j*vecViewPort.w)*vecSkill1.x);
			
			Color.rgb /= 9;	
			
			float Helligkeit = dot(Color.rgb,float3(0.299f,0.587f,0.114f));
			Color.rgb = lerp(Helligkeit,Color.rgb,vecSkill1.y);
			Color.rgb += float3(0.98,0.588,0)*vecSkill1.z;
			Color.rgb -= vecSkill1.w;
			Color.rgb *= vecSkill5.x;
			
			float grain = tex2D(smpGrain,Tex+vecTime.w).r;
			Color.rgb -= grain;
			
			float random = tex2D(smpGrain,vecTime.w*0.00001).r;
			Color.rgb += tex1D(smpScratch,Tex.x+vecTime.w*vecSkill5.y+grain*100+random*100).r;
			
			return Color;
		}

		technique OldMovie
		{
			pass one
			{
				PixelShader = compile ps_2_0 OldMoviePS();
			}
		}
	";
}


//function to change the values for old movie
void PP_OldMovie_SetValues(var BlurStrength, var Saturation, var SepiaStrength, var Darken, var Contrast, var ScratchSpeed)
{
	//set the effects parameter
	PP_OldMovie_mat.skill1 = floatd(BlurStrength,100);		//100%
	PP_OldMovie_mat.skill2 = floatd(Saturation,100);		// 50%
	PP_OldMovie_mat.skill3 = floatd(SepiaStrength,100);	// 10%
	PP_OldMovie_mat.skill4 = floatd(Darken,100);				//  5%
	PP_OldMovie_mat.skill5 = floatd(Contrast,100);			//105%
	PP_OldMovie_mat.skill6 = floatd(ScratchSpeed,1000);	//	10
}


//initializes and activates the effect
void PP_OldMovie_init(VIEW* View, BMAP* GrainMap, BMAP* ScratchMap, var BlurStrength, var Saturation, var SepiaStrength, var Darken, var Contrast, var ScratchSpeed)
{
	if(d3d_shaderversion >= 2020)
	{
		PP_OldMovie_mat.skin1 = GrainMap;
		PP_OldMovie_mat.skin2 = ScratchMap;
		
		//set the effects parameter
		PP_OldMovie_SetValues(BlurStrength,Saturation,SepiaStrength,Darken,Contrast,ScratchSpeed);
		
		//create the effectchain
		PP_Add(PP_OldMovie_mat,View,NULL);
	}
}


//removes OldMovie
void PP_OldMovie_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_OldMovie_mat,View,NULL);
}