//////////////////////////////////////////////////
/////////////Postprocessing Raindrops/////////////
//////////////////////////////////////////////////
////18.03.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_Raindrops


//animated screen disortion 
MATERIAL* PP_Raindrops_mat =
{
	effect =
	"
		// Variables
		float4 vecSkill1;
		float4 vecTime;
		float4 vecViewDir;
		float4 vecSunDir;
		
		// Textures
		texture TargetMap;
		texture mtlSkin1;
		
		sampler postTex = sampler_state
		{
		   texture = <TargetMap>;
		   MinFilter = linear;
		   MagFilter = linear;
		   MipFilter = linear;
		   AddressU = Clamp;
		   AddressV = Clamp;
		};
		
		sampler bumpTex = sampler_state
		{
		   texture = <mtlSkin1>;
		   MinFilter = linear; 
		   MagFilter = linear; 
		   MipFilter = linear; 
		   AddressU = Wrap;
		   AddressV = Wrap;
		};
		
		// Pixelshader 
		float4 PS_RainDrops(float2 TexCoords : TEXCOORD0) : COLOR
		{
			float2 Temp_TexCoords = TexCoords+vecSkill1.xy*vecTime.w;
			float3 BumpColor = tex2D(bumpTex,Temp_TexCoords).rgb;
			
			// Calculate the reflection vector:
			float2 R = normalize(2 * dot(BumpColor.rg, float2(1,1)) * BumpColor.rg - float2(1,1));
			
			// Calculate the specular term:
			float Specular = pow(saturate(dot(R, float2(1,1))), 15);
			
			Temp_TexCoords = TexCoords+BumpColor.rg * 0.05f;
			float3 TexColor = tex2D(postTex,Temp_TexCoords).rgb;//*Specular;
		   
		   return float4(TexColor,1.0);
		}
		
		//
		technique tech_00
		{
		   pass pass_00
		   {
		      PixelShader = compile ps_2_0 PS_RainDrops();
		   }
		}
	";
}

//sets new values for the raindrops shader
void PP_Raindrops_SetValues(BMAP* NormalMap, var xMovement, var yMovement)
{
	if(NormalMap != NULL)
	{
		PP_Raindrops_mat.skin1 = NormalMap;
	}
	PP_Raindrops_mat.skill1 = floatv(xMovement);	//usually 0
	PP_Raindrops_mat.skill2 = floatv(yMovement); //very small around 0.01
}

//starts the raindrops effect
void PP_Raindrops_init(VIEW* View, BMAP* NormalMap, var xMovement, var yMovement)
{
	if(d3d_shaderversion >= 2020)
	{
		PP_Raindrops_SetValues(NormalMap,xMovement,yMovement);
		PP_Add(PP_Raindrops_mat,View,NULL);
	}
}

//removes Raindrops
void PP_Raindrops_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_Raindrops_mat,View,NULL);
}