//////////////////////////////////////////////////
//////////////////Depth of Field//////////////////
//////////////////////////////////////////////////
////08.03.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_fp_DoF

#ifndef PP_fp_Shared_Depth
	#include "PP_fp_Shared_Depth.c"
#endif


//a horizontal blur with downsampling
MATERIAL* PP_fp_DoF_HBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurHPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			Tex.xy *= vecSkill1.y;
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.y = Tex.y;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.x = Tex.x+i*vecViewPort.z*vecSkill1.x*vecSkill1.y;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurHPS();}}
		";
}


//a vertical blur
MATERIAL* PP_fp_DoF_VBlur_mat =
{
	effect = "
		float4 vecViewPort;
		float4 vecSkill1;
		static int blurRadius = 5;//number of px

		Texture TargetMap;
		sampler2D smpSource = sampler_state{texture=<TargetMap>;};

		float4 BlurVPS(float2 Tex:	TEXCOORD0):COLOR0
		{
			float4 Color = tex2D(smpSource,Tex.xy);
			float2 tempTex;
			tempTex.x = Tex.x;
			for(int i=-blurRadius;i<=blurRadius;i++)
			{
				tempTex.y = Tex.y+i*vecViewPort.w*vecSkill1.x;
				Color += tex2D(smpSource,tempTex.xy);
			}
			return Color/(blurRadius*2+1);
		}
		
		technique Blackout { pass one { PixelShader = compile ps_2_0 BlurVPS();}}
		";
}


//combines the blurred and the normal image depending on the depth to get the final DoF
MATERIAL* PP_fp_DoF_Combine_mat = 
{
	effect = "
	float4 vecSkill1;
	
	Texture TargetMap;
	Texture mtlSkin1;
	Texture mtlSkin2;
	sampler2D smpBlurred = sampler_state{texture=<TargetMap>;};
	sampler2D smpSource = sampler_state{texture=<mtlSkin1>;};
	sampler2D smpDepth = sampler_state{texture=<mtlSkin2>;};
	
	float4 AddPS(	float2 Tex: TEXCOORD0):COLOR0
	{
		float depth = tex2D(smpDepth,Tex).r;
		float BlendDepth = 0.0;
		if(depth > vecSkill1.y)
		{
			BlendDepth = (depth-vecSkill1.z)/vecSkill1.w;
		}else
		{
			if(depth < vecSkill1.y)
			{
				BlendDepth = (vecSkill1.y-depth)/vecSkill1.y;
			}else
			{
				BlendDepth = 0;
			}
		}
		BlendDepth = saturate(BlendDepth);
		
		float4 Color = lerp(tex2D(smpSource,Tex),tex2D(smpBlurred,Tex*vecSkill1.x),BlendDepth);
		return Color;
	}
	technique Blackout { pass one { PixelShader = compile ps_2_0 AddPS();}}
	";
}


//function to change the focal planes
void PP_fp_DoF_SetFocus(var NearPlaneDist, var FarPlaneDist, var FarFadeoutDist)
{
	//set to the new parameters
	PP_fp_DoF_Combine_mat.skill2 = floatv(NearPlaneDist);
	PP_fp_DoF_Combine_mat.skill3 = floatv(FarPlaneDist);
	PP_fp_DoF_Combine_mat.skill4 = floatv(FarFadeoutDist);
}

//function to change the values for DoF
void PP_fp_DoF_SetValues(var NearPlaneDist, var FarPlaneDist, var FarFadeoutDist, var BlurRadius, var DownsampleFaktor)
{
	//set the effects parameter
	PP_fp_DoF_Combine_mat.skill2 = floatv(NearPlaneDist);
	PP_fp_DoF_Combine_mat.skill3 = floatv(FarPlaneDist);
	PP_fp_DoF_Combine_mat.skill4 = floatv(FarFadeoutDist);
	
	PP_fp_DoF_HBlur_mat.skill1 = floatv(BlurRadius/100);
	PP_fp_DoF_HBlur_mat.skill2 = floatv(DownsampleFaktor);
	PP_fp_DoF_VBlur_mat.skill1 = PP_fp_DoF_HBlur_mat.skill1;
	
	PP_fp_DoF_Combine_mat.skill1 = floatd(1,DownsampleFaktor);
}


//initializes and activates DoF
void PP_fp_DoF_init(VIEW* View, var NearPlaneDist, var FarPlaneDist, var FarFadeoutDist, var BlurRadius, var DownsampleFaktor)
{
	if(d3d_shaderversion >= 2020)
	{
		VIEW* TempView = View;
		
		//set the effects parameter
		PP_fp_DoF_SetValues(NearPlaneDist,FarPlaneDist,FarFadeoutDist,BlurRadius,DownsampleFaktor);
		
		//store the previous stages image
		View = PP_LastView(View);
		if(PP_fp_DoF_Combine_mat.skin1 == NULL){PP_fp_DoF_Combine_mat.skin1 = bmap_createblack(screen_size.x,screen_size.y,video_depth);}
		View.bmap = PP_fp_DoF_Combine_mat.skin1;
		
		//create the effectchain
		PP_Add(PP_fp_DoF_HBlur_mat,View,NULL);
		PP_Add(PP_fp_DoF_VBlur_mat,View,NULL);
		PP_Add(PP_fp_DoF_Combine_mat,View,NULL);
		
		//assign the depthmap to the shader
		if(PP_fp_Shared_Depth_orig_view == NULL)
		{
			PP_fp_Shared_Depth_init(TempView);
		}
		PP_fp_DoF_Combine_mat.skin2 = PP_fp_Shared_Depth_orig_view.bmap;
	}
}

//removes DoF
void PP_fp_DoF_remove(VIEW* View)
{
	//remove the stages
	PP_Remove(PP_fp_DoF_HBlur_mat,View,NULL);
	PP_Remove(PP_fp_DoF_VBlur_mat,View,NULL);
	PP_Remove(PP_fp_DoF_Combine_mat,View,NULL);
	
	//remove the bmaps
	ptr_remove(PP_fp_DoF_Combine_mat.skin1);
	PP_fp_DoF_Combine_mat.skin1 = NULL;
	PP_fp_DoF_Combine_mat.skin2 = NULL;
}

//makes it fit to the new resolution
void PP_fp_DoF_Video_Switch(VIEW* View)
{
	//if DoF is active
	if(PP_fp_DoF_Combine_mat.skin1 != NULL)
	{
		//remove the effect
		PP_fp_DoF_remove(View);
		
		//and create it again:
		VIEW* TempView = View;
		
		//store the previous stages image
		View = PP_LastView(View);
		if(PP_fp_DoF_Combine_mat.skin1 == NULL){PP_fp_DoF_Combine_mat.skin1 = bmap_createblack(screen_size.x,screen_size.y,video_depth);}
		View.bmap = PP_fp_DoF_Combine_mat.skin1;
		
		//create the effectchain
		PP_Add(PP_fp_DoF_HBlur_mat,View,NULL);
		PP_Add(PP_fp_DoF_VBlur_mat,View,NULL);
		PP_Add(PP_fp_DoF_Combine_mat,View,NULL);
		
		//assign the depthmap to the shader
		if(PP_fp_Shared_Depth_orig_view == NULL)
		{
			PP_fp_Shared_Depth_init(TempView);
		}
		PP_fp_DoF_Combine_mat.skin2 = PP_fp_Shared_Depth_orig_view.bmap;
	}
}