//////////////////////////////////////////////////
////////Postprocessing SoftStencilShadows/////////
//////////////////////////////////////////////////
////05.04.2008////////////////by Nils Daumann/////
/////////////slindev.wordpress.com////////////////
//////////////////////////////////////////////////
#define PP_fp_SoftStencilShadows


//softens stencil shadows
MATERIAL* PP_fp_SoftStencilShadows_mat =
{
	effect =
	"
		float4 vecViewPort;
		float4 vecSkill1;
		
		texture TargetMap;
		texture StencilMap;
		texture mtlSkin1;
		
		sampler smpSource = sampler_state { texture = <TargetMap>; };
		sampler smpStencil = sampler_state { texture = <StencilMap>; };
		sampler smpDepth = sampler_state { texture = <mtlSkin1>; };
		
		const float3 samples[12] =
		{
			-1.0,	 0.0, 0.083,
			 1.0,	 0.0, 0.083,
			 0.0,	 1.0, 0.083,
			 0.0,	-1.0, 0.083,
			-2.0,	 0.0, 0.083,
			 2.0,	 0.0, 0.083,
			 0.0,	 2.0, 0.083,
			 0.0,	-2.0, 0.083,
			-3.0,	 0.0, 0.083,
			 3.0,	 0.0, 0.083,
			 0.0,	 3.0, 0.083,
			 0.0,	-3.0, 0.083
		};
		
		float4 blurPS( float2 Tex : TEXCOORD0) : COLOR0
		{
			float4 Color = tex2D(smpSource,Tex.xy); // sample previous render target
			float Stencil = 0;
			float fDist = saturate(1.0-tex2D(smpDepth,Tex.xy).r*vecSkill1.y)*vecSkill1.x;
			
			for(int i=0;i<12;i++)
				Stencil += samples[i].z*tex2D(smpStencil,Tex.xy+float2(samples[i].x*vecViewPort.z,samples[i].y*vecViewPort.w)*fDist).a;
			
			Color.rgb *= saturate(1-Stencil*2); // blend stencil alpha over previous render target
			return Color;
		}
		
		technique blur
		{
			pass one
			{
				PixelShader = compile ps_2_0 blurPS();
			}
		}
	";
}

//sets the effects values
void PP_fp_SoftStencilShadows_SetValues(var Blur, var Depth)
{
	PP_fp_SoftStencilShadows_mat.skill1 = floatv(Blur);
	PP_fp_SoftStencilShadows_mat.skill2 = floatv(Depth);
}

//adds the effect
void PP_fp_SoftStencilShadows_init(VIEW* View, var Blur, var Depth)
{
	if(d3d_shaderversion >= 2020)
	{
		PP_fp_SoftStencilShadows_SetValues(Blur,Depth);
		
		if(render_stencil == NULL)
		{
			render_stencil = bmap_createblack(screen_size.x,screen_size.y,32);
		}
		
		PP_Add(PP_fp_SoftStencilShadows_mat,View,NULL);
		
		//assign the depthmap to the shader
		if(PP_fp_Shared_Depth_orig_view == NULL)
		{
			PP_fp_Shared_Depth_init(View);
		}
		PP_fp_SoftStencilShadows_mat.skin1 = PP_fp_Shared_Depth_orig_view.bmap;
	}
}

//removes the effect
void PP_fp_SoftStencilShadows_remove(VIEW* View)
{
	//remove the stage
	PP_Remove(PP_fp_SoftStencilShadows_mat,View,NULL);
	
	//remove the stencilbmap
	ptr_remove(render_stencil);
	render_stencil = NULL;
}

//adjusts the effect to a new screenresolution
void PP_fp_SoftStencilShadows_Video_Switch(VIEW* View)
{
	if(render_stencil != NULL)
	{
		//remove the stage
		PP_Remove(PP_fp_SoftStencilShadows_mat,View,NULL);
		
		//remove the stencilbmap
		ptr_remove(render_stencil);
		render_stencil = NULL;
		
		//create the effect again:
		
		if(render_stencil == NULL)
		{
			render_stencil = bmap_createblack(screen_size.x,screen_size.y,24);
		}
		
		PP_Add(PP_fp_SoftStencilShadows_mat,View,NULL);
		
		//assign the depthmap to the shader
		if(PP_fp_Shared_Depth_orig_view == NULL)
		{
			PP_fp_Shared_Depth_init(View);
		}
		PP_fp_SoftStencilShadows_mat.skin1 = PP_fp_Shared_Depth_orig_view.bmap;
	}
}